package com.gradle.ares

import org.objectweb.asm.ClassVisitor
import org.objectweb.asm.FieldVisitor
import org.objectweb.asm.Label
import org.objectweb.asm.MethodVisitor
import org.objectweb.asm.Opcodes

/**
 * Created by Glen on 2017/2/8.
 */

public class MethodLifeCycleVisitor extends MethodVisitor {
    public final static String LifeClass = "com/uddream/plugin/LifeCycleDelegate";
    public final static String GetLifeMethod = "\$getLifeCycleAresDelegate";

    public final static String[] SuperClass = [
            "android/app/Activity",
            "android/support/v4/app/FragmentActivity",
            "android/support/v7/app/AppCompatActivity",
            "android/support/v4/app/Fragment",
            "android/support/v4/app/ListFragment",
            "android/support/v4/app/DialogFragment",
            "android/app/Fragment",
            "android/app/ListFragment",
            "android/app/DialogFragment"
    ] as String[]

    public final static Agent[] Modify = [
            new Agent("onCreate", "(Landroid/os/Bundle;)V"),
            new Agent("onResume", "()V"),
            new Agent("onPause", "()V"),
            new Agent("onDestroy", "()V"),
            new Agent("onSaveInstanceState", "(Landroid/os/Bundle;)V"),
            new Agent("onRestoreInstanceState", "(Landroid/os/Bundle;)V"),
            new Agent("setUserVisibleHint", "(Z)V"),
            new Agent("onHiddenChanged", "(Z)V")
    ] as Agent[]

    public final static Agent getAgent(String superName, int access, String name, String desc) {
        for (String s : SuperClass) {
            if (s.equals(superName)) {
                for (Agent item : Modify) {
                    if (item.name.equals(name) && item.desc.equals(desc)) {
                        item.access = access
                        return item
                    }
                }
                break
            }
        }
        return null
    }

    public final
    static void addMethods(Set<Agent> agentSet, String name, String superName, ClassVisitor cv) {
        addLifeCycleFieldMethod(name, cv)
        for (Agent agent : Modify) {
            if (!agentSet.contains(agent)) {
                //method
                int count = 0;
                for (def item : agent.desc.substring(1, agent.desc.indexOf(")")).split(";")) {
                    if (!item.equals("")) count++
                }

                MethodVisitor mv = cv.visitMethod(Opcodes.ACC_PUBLIC, agent.name, agent.desc, null, null)
                mv.visitCode()
                mv.visitVarInsn(Opcodes.ALOAD, 0)
                mv.visitMethodInsn(Opcodes.INVOKEVIRTUAL, name, GetLifeMethod, "()L" + LifeClass + ";", false)
                for (int i = 1; i <= count; i++) mv.visitVarInsn(Opcodes.ALOAD, i)
                mv.visitMethodInsn(Opcodes.INVOKEVIRTUAL, LifeClass, agent.name, agent.desc, false)

                mv.visitVarInsn(Opcodes.ALOAD, 0)
                for (int i = 1; i <= count; i++) mv.visitVarInsn(Opcodes.ALOAD, i)
                mv.visitMethodInsn(Opcodes.INVOKESPECIAL, superName, agent.name, agent.desc, false)
                mv.visitInsn(Opcodes.RETURN)
                mv.visitMaxs(0, 0)
                mv.visitEnd()
            }
        }
    }

    /**
     * 注入代理字段及初始化方法
     * @param cv
     */
    private final static void addLifeCycleFieldMethod(String name, ClassVisitor cv) {
        //desc
        String fieldName = "\$lifeCycleAresDelegate"
        String fieldDesc = "L" + LifeClass + ";"
        //field
        FieldVisitor fv = cv.visitField(Opcodes.ACC_PUBLIC, fieldName, fieldDesc, null, null)
        fv.visitEnd()
        //method
        MethodVisitor mv = cv.visitMethod(Opcodes.ACC_PUBLIC, GetLifeMethod, "()" + fieldDesc, null, null)
        mv.visitCode()
        mv.visitVarInsn(Opcodes.ALOAD, 0)
        mv.visitFieldInsn(Opcodes.GETFIELD, name, fieldName, fieldDesc)
        Label label = new Label()
        mv.visitJumpInsn(Opcodes.IFNONNULL, label)
        mv.visitVarInsn(Opcodes.ALOAD, 0)
        mv.visitVarInsn(Opcodes.ALOAD, 0)
        mv.visitMethodInsn(Opcodes.INVOKESTATIC, LifeClass, "create", "(Ljava/lang/Object;)" + fieldDesc, false)
        mv.visitFieldInsn(Opcodes.PUTFIELD, name, fieldName, fieldDesc)
        mv.visitLabel(label)
        mv.visitVarInsn(Opcodes.ALOAD, 0)
        mv.visitFieldInsn(Opcodes.GETFIELD, name, fieldName, fieldDesc)
        mv.visitInsn(Opcodes.ARETURN)
        mv.visitMaxs(0, 0)
        mv.visitEnd()
    }

    private int pCount;
    private String name;//class name
    private Agent agent;

    MethodLifeCycleVisitor(MethodVisitor mv, String name, Agent agent) {
        super(Opcodes.ASM5, mv)
        this.agent = agent
        this.name = name
        for (def item : agent.desc.substring(1, agent.desc.indexOf(")")).split(";")) {
            if (!item.equals("")) this.pCount++
        }
    }

    @Override
    void visitCode() {
        super.visitCode()
        visitVarInsn(Opcodes.ALOAD, 0)
        visitMethodInsn(Opcodes.INVOKEVIRTUAL, name, GetLifeMethod, "()L" + LifeClass + ";", false)
        for (int i = 1; i <= pCount; i++) visitVarInsn(Opcodes.ALOAD, i)
        visitMethodInsn(Opcodes.INVOKEVIRTUAL, LifeClass, agent.name, agent.desc, false)
    }
}
